@@ -50,18 +50,23 @@ class Channel(channel.Channel):
     def basic_publish(self, message, exchange, routing_key, mandatory=False,
             immediate=False):
         message_data, properties = message
-        return channel.Channel.basic_publish(self,
-                                            exchange,
-                                            routing_key,
-                                            message_data,
-                                            properties,
-                                            mandatory,
-                                            immediate)
+        try:
+            return channel.Channel.basic_publish(self,
+                                                 exchange,
+                                                 routing_key,
+                                                 message_data,
+                                                 properties,
+                                                 mandatory,
+                                                 immediate)
+        finally:
+            self.handler.connection.flush_outbound()
 
     def basic_consume(self, queue, no_ack=False, consumer_tag=None,
             callback=None, nowait=False):
 
         def _callback_decode(channel, method, header, body):
+            print("IN CALLBACK DECODE: %r" % (
+                channel, method, header, body))
             return callback((channel, method, header, body))
 
         return channel.Channel.basic_consume(self, _callback_decode,
@@ -81,11 +86,18 @@ class Channel(channel.Channel):
 
     def message_to_python(self, raw_message):
         """Convert encoded message body back to a Python value."""
+        print("RAW MESSAGE: %r" % (raw_message, ))
         return self.Message(channel=self, amqp_message=raw_message)
 
     def basic_ack(self, delivery_tag):
         return channel.Channel.basic_ack(self, delivery_tag)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *exc_info):
+        self.close()
+
 
 class BlockingConnection(blocking_adapter.BlockingConnection):
 
