@@ -14,10 +14,12 @@ import socket
 
 from itertools import count
 from multiprocessing.util import Finalize
+from time import sleep
 from Queue import Empty
 
 from kombu.transport import base
-from kombu.utils import emergency_dump_state, OrderedDict, say
+from kombu.utils import emergency_dump_state, say
+from kombu.utils.compat import OrderedDict
 
 from kombu.transport.virtual.scheduling import FairCycle
 from kombu.transport.virtual.exchange import STANDARD_EXCHANGE_TYPES
@@ -79,7 +81,7 @@ class QoS(object):
 
         """
         pcount = self.prefetch_count
-        return (not pcount or len(self._delivered) > pcount)
+        return (not pcount or len(self._delivered) < pcount)
 
     def append(self, message, delivery_tag):
         """Append message to transactional state."""
@@ -103,7 +105,7 @@ class QoS(object):
         while delivered:
             try:
                 _, message = delivered.popitem()
-            except KeyError:
+            except KeyError:  # pragma: no cover
                 break
 
             try:
@@ -143,13 +145,14 @@ class Message(base.Message):
 
     def __init__(self, channel, payload, **kwargs):
         properties = payload["properties"]
-        fields = {"body": payload.get("body").encode("utf-8"),
+        fields = {"body": payload.get("body"),
                   "delivery_tag": properties["delivery_tag"],
                   "content_type": payload.get("content-type"),
                   "content_encoding": payload.get("content-encoding"),
                   "headers": payload.get("headers"),
                   "properties": properties,
-                  "delivery_info": properties.get("delivery_info")}
+                  "delivery_info": properties.get("delivery_info"),
+                  "postencode": "utf-8"}
         super(Message, self).__init__(channel, **dict(kwargs, **fields))
 
     def serializable(self):
@@ -169,7 +172,7 @@ class AbstractChannel(object):
 
     """
 
-    def _get(self, queue):
+    def _get(self, queue, timeout=None):
         """Get next message from `queue`."""
         raise NotImplementedError("Virtual channels must implement _get")
 
@@ -204,9 +207,9 @@ class AbstractChannel(object):
         """
         pass
 
-    def _poll(self, queues):
+    def _poll(self, cycle, timeout=None):
         """Poll a list of queues for available messages."""
-        return FairCycle(self._get, queues, Empty).get()
+        return cycle.get()
 
 
 class Channel(AbstractChannel):
@@ -234,8 +237,10 @@ class Channel(AbstractChannel):
     def __init__(self, connection, **kwargs):
         self.connection = connection
         self._consumers = set()
+        self._cycle = None
         self._tag_to_queue = {}
         self._qos = None
+        self.closed = False
 
         # instantiate exchange types
         self.exchange_types = dict((typ, cls(self))
@@ -265,7 +270,7 @@ class Channel(AbstractChannel):
 
     def exchange_delete(self, exchange, if_unused=False, nowait=False):
         """Delete `exchange` and all its bindings."""
-        for rkey, _, queue in self.typeof(exchange).get_table():
+        for rkey, _, queue in self.get_table(exchange):
             self.queue_delete(queue, if_unused=True, if_empty=True)
         self.state.exchanges.pop(exchange, None)
 
@@ -321,13 +326,14 @@ class Channel(AbstractChannel):
 
         self.connection._callbacks[queue] = _callback
         self._consumers.add(consumer_tag)
+        self._reset_cycle()
 
     def basic_cancel(self, consumer_tag):
         """Cancel consumer by consumer tag."""
         self._consumers.remove(consumer_tag)
+        self._reset_cycle()
         queue = self._tag_to_queue.pop(consumer_tag, None)
-        if queue:
-            self.connection._callbacks.pop(queue, None)
+        self.connection._callbacks.pop(queue, None)
 
     def basic_get(self, queue, **kwargs):
         """Get message by direct access (synchronous)."""
@@ -350,7 +356,8 @@ class Channel(AbstractChannel):
         """Reject message."""
         self.qos.reject(delivery_tag, requeue=requeue)
 
-    def basic_qos(self, prefetch_size, prefetch_count, apply_global=False):
+    def basic_qos(self, prefetch_size=0, prefetch_count=0,
+            apply_global=False):
         """Change QoS settings for this channel.
 
         Only `prefetch_count` is supported.
@@ -373,8 +380,8 @@ class Channel(AbstractChannel):
         Returns `default` if no queues matched.
 
         """
-        table = self.get_table(exchange)
         try:
+            table = self.get_table(exchange)
             return self.typeof(exchange).lookup(table, exchange,
                                                 routing_key, default)
         except KeyError:
@@ -394,7 +401,7 @@ class Channel(AbstractChannel):
         if self._consumers and self.qos.can_consume():
             if hasattr(self, "_get_many"):
                 return self._get_many(self._active_queues, timeout=timeout)
-            return self._poll(self._active_queues)
+            return self._poll(self.cycle, timeout=timeout)
         raise Empty()
 
     def message_to_python(self, raw_message):
@@ -429,9 +436,20 @@ class Channel(AbstractChannel):
     def close(self):
         """Close channel, cancel all consumers, and requeue unacked
         messages."""
+        self.closed = True
         map(self.basic_cancel, list(self._consumers))
         self.qos.restore_unacked_once()
         self.connection.close_channel(self)
+        self._cycle = None
+
+    def _reset_cycle(self):
+        self._cycle = FairCycle(self._get, self._active_queues, Empty)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *exc_info):
+        self.close()
 
     @property
     def state(self):
@@ -449,6 +467,12 @@ class Channel(AbstractChannel):
     def _active_queues(self):
         return [self._tag_to_queue[tag] for tag in self._consumers]
 
+    @property
+    def cycle(self):
+        if self._cycle is None:
+            self._reset_cycle()
+        return self._cycle
+
 
 class Transport(base.Transport):
     """Virtual transport.
@@ -508,21 +532,23 @@ class Transport(base.Transport):
         while self.channels:
             try:
                 channel = self.channels.pop()
-            except KeyError:
+            except KeyError:    # pragma: no cover
                 pass
             else:
                 channel.close()
 
     def drain_events(self, connection, timeout=None):
-        cycle_seconds = len(self.channels) * self.interval
+        if not self.channels:
+            raise ValueError("No channels to drain events from.")
         loop = 0
         while 1:
             try:
                 item, channel = self.cycle.get()
             except Empty:
-                if timeout and cycle_seconds * loop >= timeout:
+                if timeout and loop * 0.1 >= timeout:
                     raise socket.timeout()
                 loop += 1
+                sleep(0.1)
             else:
                 break
 
@@ -536,4 +562,4 @@ class Transport(base.Transport):
         self._callbacks[queue](message)
 
     def _drain_channel(self, channel):
-        return channel.drain_events(timeout=self.interval)
+        return channel.drain_events()
